@@ -1,8 +1,8 @@
-from typing import Union
 import os
 from numbers import Number
 import h5py
 import numpy as np
+import nrrd
 from numpy.lib.mixins import NDArrayOperatorsMixin
 
 import tifffile
@@ -25,45 +25,75 @@ class Chunk(NDArrayOperatorsMixin):
     and `examples<https://docs.scipy.org/doc/numpy/user/basics.dispatch.html#module-numpy.doc.dispatch>`_.
 
     :param array: the data array chunk in a big dataset
-    :param global_offset: the offset of this array chunk
+    :param voxel_offset: the offset of this array chunk. 3 numbers: z, y, x
+    :param voxel_size: the size of each voxel, normally with unit of nm. 3 numbers: z, y, x.
     :return: a new chunk with array data and global offset
     """
-    def __init__(self, array, global_offset: tuple = None):
+    def __init__(self, array: np.ndarray, voxel_offset: tuple = None, voxel_size: tuple = None):
         assert isinstance(array, np.ndarray) or isinstance(array, Chunk)
         self.array = array
-        if global_offset is None:
+        if voxel_offset is None:
             if isinstance(array, Chunk):
                 self.array = array.array
-                global_offset = array.global_offset
+                voxel_offset = array.voxel_offset
             else:
-                global_offset = tuple(np.zeros(array.ndim, dtype=np.int))
-        self.global_offset = global_offset
-        assert array.ndim == len(global_offset)
+                voxel_offset = (0, 0, 0)
+        
+        if voxel_offset is not None:
+            if len(voxel_offset) == 4:
+                assert voxel_offset[0] == 0
+                voxel_offset = voxel_offset[1:]
+            assert len(voxel_offset) == 3
+
+        self.voxel_offset = voxel_offset
+        self.voxel_size = voxel_size
+        if voxel_size is not None:
+            assert len(voxel_size) == 3
+            assert np.alltrue([vs > 0 for vs in voxel_size])
+        
+        assert array.ndim >= 3 and array.ndim <= 4
         
     # One might also consider adding the built-in list type to this
     # list, to support operations like np.add(array_like, list)
     _HANDLED_TYPES = (np.ndarray, Number)
     
     @classmethod
-    def from_bbox(cls, array: np.ndarray, bbox: Bbox):
+    def from_array(cls, array: np.ndarray, bbox: Bbox, voxel_size: tuple = None):
         """
         :param array: ndarray data
-        :param bbox: cloudvolume bounding box 
+        :param bbox: cloudvolume bounding box
+        :param voxel_size: physical size of each voxel.
         :return: construct a new Chunk
         """
-        global_offset = (bbox.minpt.z, bbox.minpt.y, bbox.minpt.x)
-        return cls(array, global_offset=global_offset)
+        return cls(array, voxel_offset=bbox.minpt, voxel_size=voxel_size)
     
     @classmethod
-    def from_bbox(cls, bbox: Bbox, dtype: type = np.uint8):
+    def from_bbox(cls, bbox: Bbox, dtype: type = np.uint8,
+            voxel_size: tuple=None, all_zero: bool=False):
+        """
+        :param bbox: bounding box of chunk.
+        :param dtype: numpy data type
+        :param all_zero: is it all zero or with a fancy function
+        :return: a new chunk
+        """
         assert isinstance(bbox, Bbox)
         size = bbox.maxpt - bbox.minpt
-        return cls.create(size=size, dtype=dtype, voxel_offset=bbox.minpt)
+        return cls.create(size=size, dtype=dtype, voxel_offset=bbox.minpt,
+            voxel_size=voxel_size, all_zero=all_zero)
 
     @classmethod
     def create(cls, size: tuple = (64, 64, 64),
                dtype: type = np.uint8, voxel_offset: tuple = (0, 0, 0),
+               voxel_size: tuple = None,
                all_zero: bool = False):
+        """
+        :param size: size of chunk
+        :param dtype: numpy data type
+        :param voxel_offset: offset of the upper left corner
+        :param voxel_size: physical size of each voxel.
+        :param all_zero: is it all zero or create using a function
+        :return: a new chunk
+        """
         if isinstance(dtype, str):
             dtype = np.dtype(dtype)
 
@@ -72,7 +102,7 @@ class Chunk(NDArrayOperatorsMixin):
         else:
             ix, iy, iz = np.meshgrid(*[np.linspace(0, 1, n) for 
                                        n in size[-3:]], indexing='ij')
-            chunk = np.abs(np.sin(4 * (ix + iy)))
+            chunk = np.abs(np.sin(4 * (ix + iy + iz)))
             if len(size) == 4:
                 chunk = np.expand_dims(chunk, axis=0)
                 chunk = np.repeat(chunk, size[0], axis=0)
@@ -86,16 +116,42 @@ class Chunk(NDArrayOperatorsMixin):
             else:
                 raise NotImplementedError()
 
-        return cls(chunk, global_offset=voxel_offset)
+        return cls(chunk, voxel_offset=voxel_offset, voxel_size=voxel_size)
 
     @classmethod
-    def from_tif(cls, file_name: str, global_offset: tuple=None, dtype: str = None):
-        arr = tifffile.imread(file_name)
+    def from_nrrd(cls, file_name: str, voxel_offset: tuple=None, dtype: str = None,
+            voxel_size: tuple=None):
+        arr, _ = nrrd.read(file_name)
+
         if dtype:
             arr = arr.astype(dtype)
-        return cls(arr, global_offset=global_offset)
+        return cls(arr, voxel_offset=voxel_offset, voxel_size=voxel_size)
     
-    def to_tif(self, file_name: str=None, global_offset: tuple=None):
+    def to_nrrd(self, file_name: str=None):
+        if file_name is None:
+            file_name = f'{self.bbox.to_filename()}.nrrd'
+        print('write chunk to file: ', file_name)
+
+        if self.array.dtype==np.float32:
+            # visualization in float32 is not working correctly in ImageJ
+            # this might not work correctly if you want to save the image as it is!
+            print(yellow('transforming data type from float32 to uint8'))
+            img = self.array*255 
+            img = img.astype( np.uint8 )
+        else:
+            img = self.array
+        nrrd.write(file_name, img)
+
+    @classmethod
+    def from_tif(cls, file_name: str, voxel_offset: tuple=None, dtype: str = None,
+            voxel_size: tuple=None):
+        arr = tifffile.imread(file_name)
+
+        if dtype:
+            arr = arr.astype(dtype)
+        return cls(arr, voxel_offset=voxel_offset, voxel_size=voxel_size)
+    
+    def to_tif(self, file_name: str=None):
         if file_name is None:
             file_name = f'{self.bbox.to_filename()}.tif'
         print('write chunk to file: ', file_name)
@@ -114,58 +170,60 @@ class Chunk(NDArrayOperatorsMixin):
     def from_h5(cls, file_name: str,
                 voxel_offset: tuple=None,
                 dataset_path: str = None,
-                cutout_start: tuple=None,
-                cutout_stop: tuple=None,
-                cutout_size: tuple=None):
+                voxel_size: tuple = None,
+                cutout_start: tuple = None,
+                cutout_stop: tuple = None,
+                cutout_size: tuple = None):
+        if cutout_start is not None and cutout_size is not None:
+            cutout_stop = tuple(t+s for t, s in zip(cutout_start, cutout_size))
 
-        assert os.path.exists(file_name)
-        assert h5py.is_hdf5(file_name)
+        if not h5py.is_hdf5(file_name):
+            assert cutout_start is not None 
+            assert cutout_stop is not None
+            bbox = Bbox.from_list([*cutout_start, *cutout_stop])
+            file_name += f'{bbox.to_filename()}.h5'
 
-        voxel_offset_path = os.path.join(os.path.dirname(file_name),
-                                       'global_offset')
         with h5py.File(file_name, 'r') as f:
             if dataset_path is None:
                 for key in f.keys():
-                    if 'offset' not in key:
+                    if 'offset' not in key and 'unique' not in key:
                         # the first name without offset inside
                         dataset_path = key
+                        break
             dset = f[dataset_path]
             if voxel_offset is None: 
-                if voxel_offset_path in f:
-                    voxel_offset = tuple(f[voxel_offset_path])
+                if 'voxel_offset' in f:
+                    voxel_offset = tuple(f['voxel_offset'])
                 else:
-                    voxel_offset = tuple(0 for _ in range(dset.ndim))
+                    voxel_offset = (0, 0, 0)
+
+            if voxel_size is None:
+                if 'voxel_size' in f:
+                    voxel_size = tuple(f['voxel_size'])
+                else:
+                    voxel_size = (1, 1, 1)
 
             if cutout_start is None:
                 cutout_start = voxel_offset
+            if cutout_size is None:
+                cutout_size = dset.shape[-3:]
+            if cutout_stop is None:
+                cutout_stop = tuple(t+s for t, s in zip(cutout_start, cutout_size))
 
             for c, v in zip(cutout_start, voxel_offset):
                 assert c >= v, "can only cutout after the global voxel offset."
 
-            if cutout_start is not None:
-                if cutout_stop is None:
-                    if cutout_size is None:
-                        cutout_size = dset.shape
-                    cutout_stop = tuple(t+s for t, s in zip(cutout_start, cutout_size))
-
-                if dset.ndim == 3:
-                    dset = dset[
-                        cutout_start[0]-voxel_offset[0]:cutout_stop[0]-voxel_offset[0],
-                        cutout_start[1]-voxel_offset[1]:cutout_stop[1]-voxel_offset[1],
-                        cutout_start[2]-voxel_offset[2]:cutout_stop[2]-voxel_offset[2],
-                    ]
-                elif dset.ndim == 4:
-                    assert len(cutout_start) == 4
-                    assert len(cutout_stop) == 4
-                    dset = dset[
-                        cutout_start[0]-voxel_offset[0]:cutout_stop[0]-voxel_offset[0],
-                        cutout_start[1]-voxel_offset[1]:cutout_stop[1]-voxel_offset[1],
-                        cutout_start[2]-voxel_offset[2]:cutout_stop[2]-voxel_offset[2],
-                        cutout_start[3]-voxel_offset[3]:cutout_stop[3]-voxel_offset[3],
-                    ]
+            assert len(cutout_start) == 3
+            assert len(cutout_stop) == 3
+            dset = dset[...,
+                cutout_start[0]-voxel_offset[0]:cutout_stop[0]-voxel_offset[0],
+                cutout_start[1]-voxel_offset[1]:cutout_stop[1]-voxel_offset[1],
+                cutout_start[2]-voxel_offset[2]:cutout_stop[2]-voxel_offset[2],
+            ]
+                    
         
-        print('read from HDF5 file: {} and start with {}, ends with {}, size is {}'.format(
-            file_name, cutout_start, cutout_stop, cutout_size))
+        print(f"""read from HDF5 file: {file_name} and start with {cutout_start}, \
+ends with {cutout_stop}, size is {cutout_size}, voxel size is {voxel_size}.""")
         arr = np.asarray(dset)
         if arr.dtype == np.dtype('<f4'):
             arr = arr.astype('float32')
@@ -174,20 +232,42 @@ class Chunk(NDArrayOperatorsMixin):
 
         print('new chunk voxel offset: {}'.format(cutout_start))
 
-        return cls(arr, global_offset=cutout_start)
+        return cls(arr, voxel_offset=cutout_start, voxel_size=voxel_size)
 
-    def to_h5(self, file_name: str, with_offset: bool=True, compression="gzip"):
-        assert '.h5' in file_name
+    def to_h5(self, file_name: str, with_offset: bool=True, 
+                chunk_size: tuple=(64,64,64),
+                with_unique: bool= True, compression="gzip"):
+        """
+        :param file_name: output file name. If it is not end with h5, the coordinate will be appended to the file name.
+        :param with_offset: save the voxel offset or not
+        :param with_unique: if this is a segmentation chunk, save the unique object ids or not.
+        :param compression: use HDF5 compression or not. Options are gzip, lzf
+        """
+        if chunk_size:
+            assert len(chunk_size) == 3
+
+        if not file_name.endswith('.h5'):
+            file_name += self.bbox.to_filename() + '.h5'
 
         print('write chunk to file: ', file_name)
         if os.path.exists(file_name):
+            print(yellow(f'deleting existing file: {file_name}'))
             os.remove(file_name)
 
+
         with h5py.File(file_name, 'w') as f:
-            f.create_dataset('/main', data=self.array, compression=compression)
-            if with_offset:
-                f.create_dataset('/global_offset', data=self.global_offset)
-    
+            f.create_dataset('/main', data=self.array, chunks=chunk_size, compression=compression)
+            if self.voxel_size:
+                f.create_dataset('/voxel_size', data=self.voxel_size)
+            if with_offset and self.voxel_offset is not None:
+                f.create_dataset('/voxel_offset', data=self.voxel_offset)
+
+            if with_unique and self.is_segmentation:
+                unique = np.unique(self.array)
+                if unique[0]:
+                    unique = unique[1:]
+                f.create_dataset('/unique_nonzeros', data = unique)
+
     def __array__(self):
         return self.array
     
@@ -216,7 +296,7 @@ class Chunk(NDArrayOperatorsMixin):
         
         if type(result) is tuple:
             # multiple return values
-            return tuple(type(self)(x, global_offset=self.global_offset) for x in result)
+            return tuple(type(self)(x, voxel_offset=self.voxel_offset) for x in result)
         elif method == 'at':
             # no return value
             return None
@@ -224,7 +304,7 @@ class Chunk(NDArrayOperatorsMixin):
             return result
         elif isinstance(result, np.ndarray):
             # one return value
-            return type(self)(result, global_offset=self.global_offset)
+            return type(self)(result, voxel_offset=self.voxel_offset)
         else:
             return result
 
@@ -235,12 +315,12 @@ class Chunk(NDArrayOperatorsMixin):
         self.array[key] = value
 
     def __repr__(self):
-        return f'array: {self.array}\n global offset: {self.global_offset}'
+        return f'array: {self.array}\n global offset: {self.voxel_offset}'
     
     def __eq__(self, value):
         if isinstance(value, type(self)):
             return np.array_equal(self.array, value.array) and np.array_equal(
-                self.global_offset, value.global_offset)
+                self.voxel_offset, value.voxel_offset)
         elif isinstance(value, Number):
             return np.all(self.array==value)
         elif isinstance(value, np.ndarray):
@@ -248,13 +328,29 @@ class Chunk(NDArrayOperatorsMixin):
         else:
             raise NotImplementedError
 
+    def set_properties(self, properties: dict):
+        if 'voxel_offset' in properties:
+            self.voxel_offset = properties['voxel_offset']
+
+        if 'voxel_size' in properties:
+            self.voxel_size = properties['voxel_size']
+
+    @property
+    def properties(self) -> dict:
+        props = dict()
+        if self.voxel_offset is not None or self.voxel_offset != (0, 0, 0):
+            props['voxel_offset'] = self.voxel_offset
+        if self.voxel_size is not None or self.voxel_size != (1, 1, 1):
+            props['voxel_size'] = self.voxel_size
+        return props 
+
     @property
     def slices(self) -> tuple:
         """
         :getter: the global slice in the big volume
         """
         return tuple(
-            slice(o, o + s) for o, s in zip(self.global_offset, self.shape))
+            slice(o, o + s) for o, s in zip(self.ndoffset, self.shape))
 
     @property
     def is_image(self) -> bool:
@@ -269,12 +365,22 @@ class Chunk(NDArrayOperatorsMixin):
     def is_affinity_map(self) -> bool:
         return self.array.ndim == 4 and self.array.dtype == np.float32
 
+    @property
+    def ndoffset(self) -> tuple:
+        """ 
+        make the voxel offset have the same dimension with array
+        """
+        if self.ndim == 4:
+            return (0, *self.voxel_offset)
+        else:
+            return self.voxel_offset
+
     @property
     def bbox(self) -> Bbox:
         """
         :getter: the cloudvolume bounding box in the big volume
         """
-        return Bbox.from_delta(self.global_offset, self.array.shape)
+        return Bbox.from_delta(self.voxel_offset, self.array.shape[-3:])
     
     @property
     def ndim(self) -> int:
@@ -287,13 +393,21 @@ class Chunk(NDArrayOperatorsMixin):
     @property 
     def dtype(self) -> np.dtype:
         return self.array.dtype 
-    
+
+    @property
+    def voxel_stop(self) -> tuple:
+        return tuple(o + s for o, s in zip(self.voxel_offset, self.shape))
+
     def astype(self, dtype: np.dtype):
         if dtype != self.array.dtype:
             new_array = self.array.astype(dtype)
-            return type(self)(new_array, global_offset=self.global_offset)
+            return type(self)(new_array, voxel_offset=self.voxel_offset)
         else:
             return self
+
+    def ascontiguousarray(self):
+        np.ascontiguousarray(self.array)
+        return self
     
     def max(self, *args, **kwargs):
         return self.array.max(*args, **kwargs)
@@ -304,8 +418,8 @@ class Chunk(NDArrayOperatorsMixin):
     def transpose(self):
         """To-Do: support arbitrary axis transpose"""
         new_array = self.array.transpose()
-        new_global_offset = self.global_offset[::-1]
-        return type(self)(new_array, global_offset=new_global_offset)
+        new_voxel_offset = self.voxel_offset[::-1]
+        return type(self)(new_array, voxel_offset=new_voxel_offset)
     
     def fill(self, x):
         self.array.fill(x)
@@ -315,8 +429,8 @@ class Chunk(NDArrayOperatorsMixin):
         assert self.array.ndim == 4
         axis = 0
         arr = np.squeeze(self, axis=0)
-        global_offset = self.global_offset[:axis] + self.global_offset[axis+1:]
-        return Chunk(arr, global_offset=global_offset)
+        voxel_offset = self.voxel_offset[:axis] + self.voxel_offset[axis+1:]
+        return Chunk(arr, voxel_offset=voxel_offset, voxel_size=self.voxel_size)
     
     # @profile(precision=0)
     def channel_voting(self):
@@ -326,13 +440,13 @@ class Chunk(NDArrayOperatorsMixin):
         np.argmax(self.array, axis=0, out=out)
         # our selected channel index start from 1
         out += 1
-        return Chunk(out, global_offset=self.global_offset[1:])
+        return Chunk(out, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
 
     def mask_using_last_channel(self, threshold: float = 0.3) -> np.ndarray:
         mask = (self.array[-1, :, :, :] < threshold)
         ret = self.array[:-1, ...]
         ret *= mask
-        return Chunk(ret, global_offset=self.global_offset)
+        return Chunk(ret, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
 
     def crop_margin(self, margin_size: tuple = None, output_bbox: Bbox=None):
 
@@ -351,9 +465,9 @@ class Chunk(NDArrayOperatorsMixin):
                                        margin_size[3]: -margin_size[3]]
             else:
                 raise ValueError('the array dimension can only by 3 or 4.')
-            global_offset = tuple(
-                o + m for o, m in zip(self.global_offset, margin_size))
-            return Chunk(new_array, global_offset=global_offset)
+            voxel_offset = tuple(
+                o + m for o, m in zip(self.voxel_offset, margin_size))
+            return Chunk(new_array, voxel_offset=voxel_offset, voxel_size=self.voxel_size)
         else:
             print('automatically crop the chunk to output bounding box.')
             assert output_bbox is not None
@@ -362,22 +476,26 @@ class Chunk(NDArrayOperatorsMixin):
     def threshold(self, threshold: float):
         seg = self > threshold
         if seg.ndim == 4:
-            global_offset = self.global_offset
+            voxel_offset = self.voxel_offset
             assert seg.shape[0] == 1
             seg = seg.array[0, ...]
-            global_offset = global_offset[1:]
-            seg = Chunk(seg, global_offset = global_offset)
+            voxel_offset = voxel_offset[1:]
+            seg = Chunk(seg, voxel_offset = voxel_offset, voxel_size=self.voxel_size)
         # neuroglancer do not support bool datatype
         # numpy store bool as uint8 datatype, so this will not increase size.
         seg = seg.astype(np.uint8)
         return seg
     
-    def connected_component(self, threshold: float = 0.5, 
-                            connectivity: int = 26):
+    def connected_component(self, threshold: float = None, 
+                            connectivity: int = 6):
         """threshold the map chunk and get connected components."""
-        seg = self.threshold(threshold)
-        seg = cc3d.connected_components(seg.array, connectivity=connectivity)
-        return Chunk(seg, global_offset=self.global_offset)
+        if not self.is_segmentation and threshold is not None:
+            seg = self.threshold(threshold)
+            seg = seg.array
+        else:
+            seg = self.array 
+        seg = cc3d.connected_components(seg, connectivity=connectivity)
+        return Chunk(seg, voxel_offset=self.voxel_offset, voxel_size=self.voxel_size)
 
     def where(self, mask: np.ndarray) -> tuple:
         """
@@ -388,7 +506,7 @@ class Chunk(NDArrayOperatorsMixin):
         """
         isinstance(mask, np.ndarray)
         assert mask.shape == self.shape
-        return tuple(i + o for i, o in zip(np.where(mask), self.global_offset))
+        return tuple(i + o for i, o in zip(np.where(mask), self.voxel_offset))
 
     def add_overlap(self, other):
         """
@@ -412,8 +530,8 @@ class Chunk(NDArrayOperatorsMixin):
             slices = (slice(0, self.shape[0]), ) + slices
         internalSlices = self._get_internal_slices(slices)
         arr = self.array[internalSlices]
-        global_offset = tuple(s.start for s in slices)
-        return Chunk(arr, global_offset=global_offset)
+        voxel_offset = tuple(s.start for s in slices[-3:])
+        return Chunk(arr, voxel_offset=voxel_offset, voxel_size=self.voxel_size)
 
     def save(self, patch):
         """
@@ -430,33 +548,50 @@ class Chunk(NDArrayOperatorsMixin):
         """
         internal_slices = tuple(
             slice(max(s.start - o, 0), min(s.stop - o, h)) for s, o, h in 
-            zip(patch.slices, self.global_offset, self.shape)
+            zip(patch.slices, self.ndoffset, self.shape)
         )
         shape = (s.stop - s.start for s in internal_slices)
         patch_starts = (
             i.start - s.start + o for s, o, i in 
-            zip(patch.slices, self.global_offset, internal_slices)
+            zip(patch.slices, self.ndoffset, internal_slices)
         )
         patch_slices = tuple(slice(s, s+h) for s, h in zip(patch_starts, shape))
 
         self.array[internal_slices] += patch.array[patch_slices]
 
+    def maskout(self, chunk):
+        """ Make part of the chunk to be black.
+        """
+        assert chunk.voxel_size
+        assert self.voxel_size
+        # the voxel size should be divisible
+        div = tuple(s%c==0 for s, c in zip(self.voxel_size, chunk.voxel_size))
+        assert np.alltrue(div)
+
+        factor = tuple(s//c for s, c in zip(self.voxel_size, chunk.voxel_size))
+        for offset in np.ndindex(factor):
+            chunk.array[
+                ...,
+                np.s_[offset[0]::factor[0]],
+                np.s_[offset[1]::factor[1]],
+                np.s_[offset[2]::factor[2]]] *= self.array
+
     def _get_overlap_slices(self, other_slices):
         return tuple(
             slice(max(s1.start, s2.start), min(s1.stop, s2.stop))
             for s1, s2 in zip(self.slices, other_slices))
 
     def _get_internal_slices(self, slices):
+        assert self.ndim == len(slices)
         return tuple(
             slice(s.start - o, s.stop - o)
-            for s, o in zip(slices, self.global_offset))
+            for s, o in zip(slices, self.ndoffset))
 
 
-    def validate(self, verbose: bool = False):
+    def validate(self):
         """validate the completeness of this chunk, there
         should not have black boxes.
 
-        :param verbose: show detailed info or not
         """
-        validate_by_template_matching(self.array, verbose=verbose)
+        validate_by_template_matching(self.array)
 
