@@ -11,8 +11,8 @@ class Image(Chunk):
     """
     a chunk of image volume.
     """
-    def __init__(self, array: np.ndarray, global_offset=None):
-        super().__init__(array, global_offset=global_offset)
+    def __init__(self, array: np.ndarray, voxel_offset=None):
+        super().__init__(array, voxel_offset=voxel_offset)
 
     def inference(self, inferencer: Inferencer):
         """run convolutional net inference for this image chunk"""
