@@ -6,5 +6,5 @@ from chunkflow.chunk.image import Image
 
 def test_image_construction():
     arr = np.random.randint(256, size=(3,4,5), dtype=np.uint8)
-    image = Image(arr, global_offset=(-1,-1,-1))
+    image = Image(arr, voxel_offset=(-1,-1,-1))
  
