@@ -4,10 +4,10 @@ from chunkflow.chunk.segmentation import Segmentation
 
 def test_segmentation():
     arr = np.random.randint(256, size=(30,40,50), dtype=np.uint32)
-    seg = Segmentation(arr, global_offset=(-1,-1,-1))
+    seg = Segmentation(arr, voxel_offset=(-1,-1,-1))
 
     arr2 = arr.astype(np.uint64)
-    seg2 = Segmentation(arr2, global_offset=(-1,-1,-1))
+    seg2 = Segmentation(arr2, voxel_offset=(-1,-1,-1))
 
 
     print('compare two segments.')
